/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

// This is an automatically generated file.

#ifndef YARP_ROSMSG_deprecated1_nav_msgs_GetPlan_h
#define YARP_ROSMSG_deprecated1_nav_msgs_GetPlan_h

#include <yarp/conf/system.h>

YARP_COMPILER_DEPRECATED_WARNING(<nav_msgs_GetPlan.h> header is deprecated. Use <yarp/rosmsg/nav_msgs/GetPlan.h> instead)

#include <yarp/rosmsg/nav_msgs/GetPlan.h>
#include <geometry_msgs_PoseStamped.h>
#include <yarp/conf/api.h>

YARP_DEPRECATED_MSG("nav_msgs_GetPlan is deprecated, use yarp::rosmsg::nav_msgs::GetPlan instead")
typedef yarp::rosmsg::nav_msgs::GetPlan nav_msgs_GetPlan;

#endif // YARP_ROSMSG_deprecated1_nav_msgs_GetPlan_h
