/// Licensed to the Apache Software Foundation (ASF) under one
/// or more contributor license agreements. See the NOTICE file
/// distributed with this work for additional information
/// regarding copyright ownership. The ASF licenses this file
/// to you under the Apache License, Version 2.0 (the
/// "License"); you may not use this file except in compliance
/// with the License. You may obtain a copy of the License at
///
/// http://www.apache.org/licenses/LICENSE-2.0
///
/// Unless required by applicable law or agreed to in writing,
/// software distributed under the License is distributed on an
/// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
/// KIND, either express or implied. See the License for the
/// specific language governing permissions and limitations
/// under the License.

part of thrift;

class TTransportErrorType {
  static const int UNKNOWN = 0;
  static const int NOT_OPEN = 1;
  static const int ALREADY_OPEN = 2;
  static const int TIMED_OUT = 3;
  static const int END_OF_FILE = 4;
}

class TTransportError extends TError {
  TTransportError([int type = TTransportErrorType.UNKNOWN, String message = ""])
      : super(type, message);
}
